@@ -109,6 +109,8 @@ typedef struct msid_machine
 
   unsigned int dump;
   unsigned int secs;
+
+  unsigned int channels;
   
 } msid_machine;
 
@@ -170,10 +172,8 @@ void *player_thread(void *arg)
 
   memset(&sfinfo, 0, sizeof(SF_INFO));
 
-  // TODO - use mono/stereo support here also
-
   sfinfo.samplerate = 44100;
-  sfinfo.channels = 2;
+  sfinfo.channels = m->channels;
   sfinfo.format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
 
 #ifdef HILDON
@@ -394,6 +394,8 @@ init_msid_machine (msid_machine *m)
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(output_mode_toggle)) == TRUE) {
     channels++;
   }
+
+  m->channels = channels;
   
   m->driver = (msid_driver *) audio_iface.plugin_object;
 
@@ -626,8 +628,12 @@ play_stop_song (GtkWidget *widget,
   case READY :
 
     msid_state = PLAY;
-    gtk_widget_set_sensitive(player_play_button, TRUE);
-    gtk_widget_set_sensitive(player_pause_button, TRUE);
+
+    if (!m->dump) {
+      gtk_widget_set_sensitive(player_play_button, TRUE);
+      gtk_widget_set_sensitive(player_pause_button, TRUE);
+    }
+
     break;
 
   default :
@@ -1090,8 +1096,7 @@ launch_browser (GtkLinkButton *button,
   GList *list =NULL;
 
   GnomeVFSMimeApplication* app =
-    gnome_vfs_mime_get_default_application_for_uri
-    (link_, "text/html");
+    gnome_vfs_mime_get_default_application_for_uri(link_, "text/html");
 
   list = g_list_append(list, g_strdup (link_));
   gnome_vfs_mime_application_launch(app, list);
@@ -1138,15 +1143,28 @@ show_info_dialog (const char *title, const char *txt1, const char *txt2)
 }
 
 
+typedef struct _dump_size_args
+{
+  GtkWidget *label;
+  msid_machine *m;
+} dump_size_args;
+
 
 void calculate_size (GtkRange *range, gpointer data)
 {
-  GtkWidget *label = GTK_WIDGET(data);
+  dump_size_args *args = (dump_size_args *) data;
   char buffer[256];
+  int secs = gtk_range_get_value(range);
+  int channels = 1;
 
-  snprintf(buffer, 256, "~%2.1fMB", (gtk_range_get_value(range) * 204.8)/1024);
-  gtk_label_set_text(GTK_LABEL(label), buffer);
+  // stereo output?
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(output_mode_toggle)) == TRUE) {
+    channels++;
+  }
 
+  snprintf(buffer, 256, "~%2.1fMB", (44.1 * channels * (16/8) * secs) / 1024);
+
+  gtk_label_set_text(GTK_LABEL(args->label), buffer);
 }
 
 void
@@ -1168,9 +1186,10 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
   GtkWidget *sizelabel;
   GtkWidget *scale = gtk_hscale_new_with_range(1.0, 120.0, 1.0);
 
+  dump_size_args args;
+
   gtk_range_set_value(GTK_RANGE(scale), m->secs);
 
-  // TODO - show progress indication during dump thread
   // TODO - set as current ringtone automatically if possible?
 
   // cannot playback while dumping
@@ -1199,7 +1218,7 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
   m->song = filepath;
   m->song_no = SUBSONG;
 
-  snprintf(buffer, 256, "Msid will create\n%s.wav\nLength in secs :", name);
+  snprintf(buffer, 256, "Msid will create\n%s.wav\nLength in secs :", filepath);
   label = gtk_label_new("");
 
   gtk_label_set_markup (GTK_LABEL(label), buffer);
@@ -1207,10 +1226,13 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
   
   sizelabel = gtk_label_new("");
 
-  calculate_size(GTK_RANGE(scale), sizelabel);
+  args.label = sizelabel;
+  args.m = m;
+
+  calculate_size(GTK_RANGE(scale), &args);
   
   g_signal_connect(G_OBJECT (scale), "value-changed",
-		   G_CALLBACK (calculate_size), sizelabel);
+		   G_CALLBACK (calculate_size), &args);
 
   dialog = gtk_dialog_new_with_buttons ("Create ringtone",
 					GTK_WINDOW(app_window),
@@ -1224,7 +1246,7 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), scale);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), sizelabel);
 
-  gtk_widget_show_all (dialog);
+  gtk_widget_show_all(dialog);
 
   gint result = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1238,6 +1260,44 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
 
       play_stop_song (NULL, data);
 
+      // BLOCK HERE UNTIL THREAD READY
+      {
+	unsigned int ready = 0;
+	GtkWidget *progress = gtk_progress_bar_new();
+	GtkWidget *pdialog = gtk_dialog_new_with_buttons("Creating tone, please wait!",
+							 GTK_WINDOW(app_window),
+							 (GtkDialogFlags) (GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
+							 NULL);
+	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress), 0.0);
+	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(pdialog)->vbox), progress);
+	
+	gtk_widget_show_all(pdialog);
+  
+	while (!ready) {
+
+	  G_LOCK(msid_state);
+	  if (msid_state == READY) {
+	    ready = 1;
+	  }
+	  G_UNLOCK(msid_state);
+
+	  usleep(20000);
+	  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(progress));
+
+	  while (gtk_events_pending()) {
+	    gtk_main_iteration();
+	  }
+	  
+	}
+
+	gtk_widget_hide(pdialog);
+	gtk_widget_destroy(pdialog);
+
+      }
+
+
+      update_status_text("RINGTONE CREATED", C64_FG);
+
       break;
     default :
       break;
@@ -1245,7 +1305,6 @@ create_ringtone (GtkMenuItem *menuitem, gpointer data)
 
     gtk_widget_destroy (dialog);
 
-    update_status_text("RINGTONE CREATED", C64_FG);
 }
 
 
@@ -1295,7 +1354,7 @@ show_help_dialog (GtkMenuItem *menuitem, gpointer data)
   GtkWidget *info;
   GtkWidget *hbox;
 
-  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when tapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\n<span foreground=\"#aaaaff\">Create ringtone</span> feature creates wav file to same directory as sidtune was found from.\n\nEnjoy!";
+  char txt[] = "Use <span foreground=\"#aaaaff\">Search</span> tab to download sid files, when tapped they will appear on <span foreground=\"#aaaaff\">Player</span> tab. When a song is selected in player tab, use <span foreground=\"#aaaaff\">Screenshot</span> tab to display screenshots related to the song, works with most game sid tunes.\n\n<span foreground=\"#aaaaff\">Create ringtone</span> feature creates wav file to same directory as sidtune was found from.\n\nEnjoy!\n";
 
   GtkWidget *dialog = gtk_dialog_new_with_buttons ("Help!",
 						   GTK_WINDOW(app_window),
@@ -1407,11 +1466,11 @@ class "GtkWindow" style "window"
   gtk_rc_parse_string (rc_buffer);
 }
 
+// change between mono/stereo output
 static void output_mode_changed (GtkToggleButton *togglebutton,
 				 gpointer user_data)
 {
-  // HACK - restart of the engine needed currently
-  // TODO - figure out nice way without complete restart
+  // restart of the playback needed
 
   update_status_text("OUTPUT MODE CHANGED", C64_FG);
 
